@@ -7,6 +7,7 @@ Geo::Google::PolylineEncoder - encode lat/lons to Google Maps Polylines
   use Geo::Google::PolylineEncoder;
 
   my $points = [
+                # can also take points as [lat, lon]
 		{ lat => 38.5, lon => -120.2 },
 	        { lat => 40.7, lon => -120.95 },
 	        { lat => 43.252, lon => -126.453 },
@@ -36,7 +37,7 @@ use strict;
 use warnings;
 
 use accessors qw(num_levels zoom_factor visible_threshold force_endpoints
-		 zoom_level_breaks escape_encoded_points
+		 zoom_level_breaks escape_encoded_points lons_first
 		 points dists max_dist encoded_points encoded_levels );
 use constant defaults => {
 			  num_levels  => 18,
@@ -44,8 +45,9 @@ use constant defaults => {
 			  force_endpoints => 1,
 			  escape_encoded_points => 0,
 			  visible_threshold => 0.00001,
+			  lons_first => 0,
 			 };
-our $VERSION = 0.04;
+our $VERSION = 0.05;
 
 # The constructor
 sub new {
@@ -93,14 +95,12 @@ sub reset_encoder {
 sub set_points {
     my ($self, $points) = @_;
 
-    $points->[0]->{first} = 1;
-    $points->[-1]->{last} = 1;
+    die "points must be an arrayref!" unless UNIVERSAL::isa( $points, 'ARRAY' );
 
-    # For the moment, just stick the points we were given into $self->points:
-    return $self->points( $points );
+    # Internally, points are stored as [lat, lon].  Although this is less
+    # readable, it is more efficient than using a hash.
 
-    # TODO: make a copy of the points we were given, and do some clever logic
-    # ala:
+    # Make a copy of the points we were given
     my @points;
     if (UNIVERSAL::isa($points->[0], 'HASH')) {
 	my @keys = keys %{ $points->[0] };
@@ -108,19 +108,17 @@ sub set_points {
 	my ($lon_key) = grep( /^(?:lon)|(?:lng)$/i, @keys );
 	@points = map { [$_->{$lat_key}, $_->{$lon_key}] } @$points;
     } elsif (UNIVERSAL::isa($points->[0], 'ARRAY')) {
-	if ($self->points_in_geographic_order) {
-	    while (my ($lon, $lat) = splice( @$points, 0, 2 )) {
-		push @points, [$lat, $lon];
-	    }
+	if ($self->lons_first) {
+	    @points = map {[ $_->[1], $_->[0] ]} @$points;
 	} else {
-	    while (my ($lat, $lon) = splice( @$points, 0, 2 )) {
-		push @points, [$lat, $lon];
-	    }
+	    @points = map {[ $_->[0], $_->[1] ]} @$points;
 	}
     } else {
 	die "don't know how to handle points = $points";
     }
 
+    return $self->points( \@points );
+
     return $self;
 }
 
@@ -168,6 +166,9 @@ sub calculate_distances {
 	return $self->dists( \@dists )->max_dist( $max_dist );
     }
 
+    # cache commonly used vars:
+    my $visible_threshold = $self->visible_threshold;
+
     # Iterate through all the points, and calculate their dists
 
     # Each stack element contains the index of two points representing a line
@@ -177,30 +178,35 @@ sub calculate_distances {
     while (@stack > 0) {
 	my $current = pop @stack;
 
-	# Get the two points, $A & $B:
-	my ($A, $B) = ($points->[$current->[0]], $points->[$current->[1]]);
+	# cache to save array lookups:
+	my $current_0 = $current->[0];
+	my $current_1 = $current->[1];
 
-	# Cache their lon/lats to avoid unneccessary hash lookups.
+	# Get the two points, $A & $B:
+	my ($A, $B) = ($points->[$current_0], $points->[$current_1]);
+
+	# Cache their lon/lats to avoid unneccessary array lookups.
 	# Note: we use X/Y because it's shorter, and more math-y
-	my ($Ax, $Ay, $Bx, $By) = ($A->{lon}, $A->{lat}, $B->{lon}, $B->{lat});
+	my ($Ax, $Ay, $Bx, $By) = ($A->[1], $A->[0], $B->[1], $B->[0]);
 
 	# Create a line segment between $A & $B and calculate its length
 	# Note: cache the square of the seg length for use in calcs later...
 	my $seg_length_squared = (($Bx - $Ax) ** 2 + ($By - $Ay) ** 2);
 	my $seg_length = sqrt($seg_length_squared);
+	my $seg_length_is_0 = $seg_length == 0; # cache
 
 	# Cache the deltas in x/y for calcs later:
 	my ($Bx_minus_Ax, $By_minus_Ay) = ($Bx - $Ax, $By - $Ay);
 
 	my $current_max_dist = 0;
 	my $current_max_dist_idx;
-	for (my $i = $current->[0] + 1; $i < $current->[1]; $i++) {
+	for (my $i = $current_0 + 1; $i < $current_1; $i++) {
 	    # Get the current point:
 	    my $P = $points->[$i];
 
 	    # Cache its lon/lat to avoid unneccessary hash lookups.
 	    # Note: we use X/Y because it's shorter, and more math-y
-	    my ($Py, $Px) = ($P->{lat}, $P->{lon});
+	    my ($Py, $Px) = ($P->[0], $P->[1]);
 
 	    # Compute the distance between point $P and line segment [$A, $B].
 	    # Maths borrowed from Philip Nicoletti (see below).
@@ -216,7 +222,7 @@ sub calculate_distances {
 	    #my $dist = $self->distance($points->[$i], $A, $B, $seg_length, $seg_length_squared);
 
 	    my $dist;
-	    if ($seg_length == 0) {
+	    if ($seg_length_is_0) {
 		# The line is really just a point, so calc dist between it and $P:
 		$dist = sqrt(($By - $Py) ** 2 + ($Bx - $Px) ** 2);
 	    } else {
@@ -281,6 +287,7 @@ sub calculate_distances {
 			     ($Ax - $Px) * $By_minus_Ay) / $seg_length_squared;
 		    $dist = abs($s) * $seg_length;
 		}
+		# warn "\t$Px\t$Py\t$Ax\t$Ay\t$Bx\t$By\t$r\t$dist\n";
 	    }
 
 	    # See if this distance is the greatest for this segment so far:
@@ -296,11 +303,11 @@ sub calculate_distances {
 	# If the point that had the greatest distance from the line seg is
 	# also greater than our threshold, process again using it as a new
 	# start/end point for the line.
-	if ($current_max_dist > $self->visible_threshold) {
+	if ($current_max_dist > $visible_threshold) {
 	    # store this distance - we'll use it later when creating zoom values
 	    $dists[$current_max_dist_idx] = $current_max_dist;
-	    push @stack, [$current->[0], $current_max_dist_idx];
-	    push @stack, [$current_max_dist_idx, $current->[1]];
+	    push @stack, [$current_0, $current_max_dist_idx];
+	    push @stack, [$current_max_dist_idx, $current_1];
 	}
     }
 
@@ -322,19 +329,23 @@ sub encode_points {
     my ($last_lat, $last_lon) = (0.0, 0.0);
 
     for (my $i = 0; $i < @$points; $i++) {
-	if (defined($dists->[$i]) || $i == 0 || $i == @$points - 1) {
-	    my $point = $points->[$i];
-	    my $lat = $point->{lat};
-	    my $lon = $point->{lon};
+	my $point = $points->[$i];
+	my $lat = $point->[0];
+	my $lon = $point->[1];
 
-	    # compute deltas
-	    my $delta_lat = $lat - $last_lat;
-	    my $delta_lon = $lon - $last_lon;
-	    ($last_lat, $last_lon) = ($lat, $lon);
+	if (defined($dists->[$i]) || $i == 0 || $i == @$points - 1) {
+	    # compute deltas, rounded to 5 decimal places:
+	    my $lat_e5    = sprintf('%.5f', $lat)+0; # round()
+	    my $lon_e5    = sprintf('%.5f', $lon)+0; # round()
+	    my $delta_lat = sprintf('%.5f', $lat_e5 - $last_lat)+0;
+	    my $delta_lon = sprintf('%.5f', $lon_e5 - $last_lon)+0;
+	    ($last_lat, $last_lon) = ($lat_e5, $lon_e5);
 
 	    $encoded_points .=
 	      $self->encode_signed_number($delta_lat) .
 	      $self->encode_signed_number($delta_lon);
+	} else {
+	    # warn "skipping point: $lat, $lon";
 	}
     }
 
@@ -420,10 +431,12 @@ sub compute_level {
 }
 
 # Based on the official google example
+# http://code.google.com/apis/maps/documentation/include/polyline.js
 sub encode_signed_number {
     my ($self, $orig_num) = @_;
 
-    # Take the decimal value and multiply it by 1e5, flooring the result:
+    # 1. Take the initial signed value:
+    # 2. Take the decimal value and multiply it by 1e5, rounding the result:
 
     # Note 1: we limit the number to 5 decimal places with sprintf to avoid
     # perl's rounding errors (they can throw the line off by a big margin sometimes)
@@ -431,59 +444,140 @@ sub encode_signed_number {
     # 34.06694 - 34.06698 will give you -3.999999999999999057E-5 which doesn't
     # encode properly. -4E-5 encodes properly.
 
-    # Note 2: we use sprintf(%8.0f ...) rather than int() for similar reasons
+    # Note 2: we use sprintf(%.0f ...) rather than int() for similar reasons
     # (see perldoc -f int), though there's not much in it and the sprintf approach
     # ends up doing more of a round() than a floor() in some cases:
     #   floor = -30   num=-30 *int=-29  1e5=-30  %3.5f=-0.00030  orig=-0.000300000000009959
     #   floor = 119  *num=120  int=119  1e5=120  %3.5f=0.00120   orig=0.0011999999999972
-    # We don't use floor() to avoid a dependency on POSIX
+
+    # Note 3: We don't use floor() to avoid a dependency on POSIX.  And it
+    # doesn't round() anyway.
 
     # do this in a series of steps so we can see what's going on in the debugger:
-    my $num3_5 = sprintf('%3.5f', $orig_num)+0;
+    my $num3_5  = sprintf('%.5f', $orig_num)+0; # round at 5 decimal places
     my $num_1e5 = $num3_5 * 1e5;
-    my $num = sprintf('%8.0f', $num_1e5)+0;
+    my $num      = sprintf('%.0f', $num_1e5)+0; # think int(...)
 
-    # my $int = int($num_1e5);
-    # my $floor = floor($num_1e5);
-    # warn "floor = $floor\tnum=$num\tint=$int\t1e5=$num_1e5\t%3.5f=$num3_5\torig=$orig_num\n"
-    #   if ($floor != $num or $num != $int);
+    # RT 49327: the signedness has to be determined *after* rounding
+    my $is_negative = $num < 0;
 
+    {
+	# 3. Convert the decimal value to binary.  Note that a negative value
+	# must be calculated using its two's complement by inverting the
+	# binary value and adding one to the result.
 
-    # Convert the decimal value to binary.
-    # Note that a negative value must be inverted and provide padded values toward the byte boundary
-    # (perl ints are already manipulatable in binary, so do nothing)
+	# Note: perl ints are already binary, but bitwise operators work on
+	# the assumption they are unsigned, ie ~$num => one's complement.
+	# if we 'use integer' bitwise operands are treated as signed:
+	use integer; # force 2's complement
 
-    # Shift the binary value:
-    $num = $num << 1;
+	# 4. Left-shift the binary value one bit:
+	$num = $num << 1;
 
-    # If the original decimal value was negative, invert this encoding:
-    if ($orig_num < 0) {
-	$num = ~$num;
+	# 5. If the original decimal value is negative, invert this encoding:
+	# (see note on RT 49327 above)
+	if ($is_negative) {
+	    $num = ~$num;
+	}
     }
 
     return $self->encode_number($num);
 }
 
 # Based on the official google example
+# http://code.google.com/apis/maps/documentation/include/polyline.js
 sub encode_number {
     my ($self, $num) = @_;
+    no integer; # treat bitwise operands as unsigned
+
+    # 6. Break the binary value out into 5-bit chunks (starting from the right hand side):
+    # 7. Place the 5-bit chunks into reverse order:
+    # 8. OR each value with 0x20 if another bit chunk follows:
+    # 9. Convert each value to decimal:
+    # 10. Add 63 to each value:
 
     my $encodeString = "";
-    my ($nextValue, $finalValue);
-
-    # Break the binary value out into 5-bit chunks (starting from the right hand side):
     while ($num >= 0x20) {
-	$nextValue = (0x20 | ($num & 0x1f)) + 63;
-	$encodeString .= chr($nextValue);
+	my $nextValue = (0x20 | ($num & 0x1f)) + 63;
+	$encodeString .= chr( $nextValue );
 	$num >>= 5;
     }
-    $finalValue = $num + 63;
-    $encodeString .= chr($finalValue);
+
+    my $finalValue = $num + 63;
+    $encodeString .= chr( $finalValue );
 
     return $encodeString;
 }
 
 
+# Decode an encoded polyline into a list of lat/lng tuples.
+# adapted from http://code.google.com/apis/maps/documentation/include/polyline.js
+sub decode_points {
+    my ($class, $encoded) = @_;
+
+    my $len = length( $encoded );
+    my @array;
+
+    my $index = 0;
+    my $lat = 0;
+    my $lon = 0;
+
+    while ($index < $len) {
+	{
+	    my $b;
+	    my $shift = 0;
+	    my $result = 0;
+	    do {
+		$b = ord( substr( $encoded, $index++, 1 ) ) - 63;
+		$result |= ($b & 0x1f) << $shift;
+		$shift += 5;
+	    } while ($b >= 0x20);
+	    my $dlat = $result >> 1;
+	    if ($result & 1) {
+		use integer; # force 2's complement
+		$dlat = ~$dlat;
+	    }
+	    $lat += $dlat;
+
+	    # cut-n-paste to improve performance?
+	    $shift = 0;
+	    $result = 0;
+	    do {
+		$b = ord( substr( $encoded, $index++, 1 ) ) - 63;
+		$result |= ($b & 0x1f) << $shift;
+		$shift += 5;
+	    } while ($b >= 0x20);
+	    my $dlon = $result >> 1;
+	    if ($result & 1) {
+		use integer; # force 2's complement
+		$dlon = ~$dlon;
+	    }
+	    $lon += $dlon;
+	}
+
+	push @array, { lat => $lat * 1e-5, lon => $lon * 1e-5 };
+    }
+
+    return \@array;
+}
+
+# Decode an encoded levels string into a list of levels.
+# adapted from http://code.google.com/apis/maps/documentation/include/polyline.js
+sub decode_levels {
+    my ($class, $encoded) = @_;
+
+    my $len = length( $encoded );
+    my @levels;
+
+    for (my $index = 0; $index < $len; $index++) {
+	my $level = ord( substr( $encoded, $index, 1 ) ) - 63;
+	push @levels, $level;
+    }
+
+    return \@levels;
+}
+
+
 1;
 
 __END__
@@ -495,8 +589,8 @@ format for use with Google Maps.  This format is described here:
 
 L<http://code.google.com/apis/maps/documentation/polylinealgorithm.html>
 
-The module is a port of Mark McClure's C<PolylineEncoder.js> with some minor
-tweaks.  The original can be found here:
+The module is a port of Mark McClure's C<PolylineEncoder.js> with some tweaks.
+The original can be found here:
 
 L<http://facstaff.unca.edu/mcmcclur/GoogleMaps/EncodePolyline/>
 
@@ -508,7 +602,7 @@ L<http://facstaff.unca.edu/mcmcclur/GoogleMaps/EncodePolyline/>
 
 Create a new encoder.  Arguments are optional and correspond to the accessor
 with the same name: L</num_levels>, L</zoom_factor>, L</visible_threshold>,
-L</force_endpoints>.
+L</force_endpoints>, etc...
 
 Note: there's nothing stopping you from setting these properties each time you
 L</encode> a polyline.
@@ -526,7 +620,7 @@ Default: 2.
 =item visible_threshold
 
 Indicates the length of a barely visible object at the highest zoom level.
-Default: 0.00001.
+Default: 0.00001.  err.. units.
 
 =item force_endpoints
 
@@ -550,6 +644,28 @@ out right, but end up horribly wrong).  It may even crash your browser.
 
 Default: 0=false.
 
+=item lons_first
+
+Specifies the order in which coordinates passed as arrayrefs to L</encode> should be
+interpreted:
+
+  # false: lat, lon
+  $encoder->encode([
+     [ 38.5, -120.2 ],
+     [ 40.7, -120.95 ],
+  ]);
+
+  # true: lon, lat
+  $encoder->encode([
+     [ -120.2, 38.5 ],
+     [ -120.95, 40.7 ],
+  ]);
+
+Default: 0 = lat,lon
+
+(Yes, the default feels wrong to the mathematician in me, but that's how Google
+Maps do it, so for sake of consistency...)
+
 =back
 
 =head1 METHODS
@@ -559,10 +675,10 @@ Default: 0=false.
 =item encode( \@points );
 
 Encode the points into a string for use with Google Maps C<GPolyline.fromEncoded>
-using a variant of the Douglas-Peucker algorithm and the Polyline encoding
-algorithm defined by Google.
+using a variant of the Douglas-Peucker algorithm to set levels, and the Polyline
+encoding algorithm defined by Google.
 
-Expects a reference to a C<@points> array ala:
+Expects a reference to a C<@points> array:
 
   [
    { lat => 38.5, lon => -120.2 },
@@ -570,6 +686,20 @@ Expects a reference to a C<@points> array ala:
    { lat => 43.252, lon => -126.453 },
   ];
 
+The individual points can also be given as arrayrefs:
+
+  [
+   [ 38.5, -120.2 ],
+   [ 40.7, -120.95 ],
+   [ 43.252, -126.453 ],
+  ];
+
+I<Note:> I tried to avoid this initially, because there's no standard for which
+should come first: I<lat>s or I<lon>s.  But I agree, it's more convenient in
+some cases so I've given you enough rope to hang yourself...  Of course you can
+easily unhang yourself:  the order for arrayrefs defaults to C<lat, lon>, but
+you can change that by setting L</lons_first>.
+
 Returns a hashref containing:
 
   {
@@ -582,17 +712,47 @@ Returns a hashref containing:
 You can then use the L<JSON> modules (or XML, or whatever) to pass the encoded
 values to your Javascript application for use there.
 
+=item decode_points( $encoded_polyline );
+
+Given an encoded polyline, returns the points:
+
+  [
+   { lat => 38.5, lon => -120.2 },
+   { lat => 40.7, lon => -120.95 },
+   { lat => 43.252, lon => -126.453 },
+  ];
+
+Note that these will likely be slightly different from the original points due
+to rounding errors during both L</encode> & decoding.
+
+=item decode_levels( $encoded_levels );
+
+Given encoded levels, returns the levels:
+
+  [ 17, 16, 17 ]
+
 =back
 
+=head1 WHY DO MY LINES LOOK FUNNY?
+
+Do your lines all go through the north pole?  Maybe you have your I<lon>s &
+I<lat>s mixed up...  If so and you're using point arrays, you can set
+L</lons_first>.
+
+Do your points not show up at particular zoom levels?  That's not a bug, it's a
+feature!  Try playing with L</visible_threshold>.
+
+Do your encoded lines cause your browser to crash?  Sounds like a bug - file
+it!
+
+=head1 BUGS
+
+L<https://rt.cpan.org/Dist/Display.html?Queue=Geo-Google-PolylineEncoder>
+
 =head1 TODO
 
-Benchmarking, & maybe bring distance calcs in-line as Joel Rosenberg did:
-L<http://facstaff.unca.edu/mcmcclur/GoogleMaps/EncodePolyline/gmap_polyline_encoder.rb.txt>
-
-As Lee Goddard suggests, accept points as arrays in inputs to encode(), eg:
-
- my $points = [ [$lat1, $lon1], ... ]; # like this
- my $points = [ $lat1, $lon1, $lat2, $lon2 ]; # or this
+More optimization: encoding big files is *slow*.  Maybe XS implementation if
+there's enough demand for it?
 
 =head1 AUTHOR
 
@@ -603,9 +763,12 @@ L<http://facstaff.unca.edu/mcmcclur/GoogleMaps/EncodePolyline/PolylineEncoderCla
 
 Some encoding ideas borrowed from L<Geo::Google>.
 
+Bringing distance calcs in-line was Joel Rosenberg's idea:
+L<http://facstaff.unca.edu/mcmcclur/GoogleMaps/EncodePolyline/gmap_polyline_encoder.rb.txt>
+
 =head1 COPYRIGHT
 
-Copyright (c) 2008 Steve Purkis.
+Copyright (c) 2008-2010 Steve Purkis.
 Released under the same terms as Perl itself.
 
 =head1 SEE ALSO
@@ -615,6 +778,6 @@ L<http://facstaff.unca.edu/mcmcclur/GoogleMaps/EncodePolyline/PolylineEncoderCla
 (JavaScript implementation),
 L<http://www.usnaviguide.com/google-encode.htm> (similar implementation in perl),
 L<Geo::Google>,
-L<JSON>
+L<JSON::Any>
 
 =cut
